@@ -1,5 +1,5 @@
 /*
-*   $Id: beta.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: beta.c,v 1.4 2002/02/19 05:18:46 darren Exp $
 *
 *   Copyright (c) 1999-2000, Mjlner Informatics
 *
@@ -20,7 +20,6 @@
 #include <string.h>
 
 #include "entry.h"
-#include "main.h"
 #include "parse.h"
 #include "read.h"
 #include "vstring.h"
@@ -107,8 +106,8 @@ static void findBetaTags (void)
 	last = vStringLength (line) - 1;
 	first = 0;
 	/* skip white space at start and end of line */
-	while (last && isspace (vStringChar (line, last))) last--;
-	while (first < last && isspace (vStringChar (line, first))) first++;
+	while (last && isspace ((int) vStringChar (line, last))) last--;
+	while (first < last && isspace ((int) vStringChar (line, first))) first++;
 	/* if line still has a reasonable length and ... */
 	if (last - first > 4 &&
 	   (vStringChar (line, first)     == '-' && 
@@ -126,17 +125,17 @@ static void findBetaTags (void)
 		last -= 2;
 		first += 2;
 		while (last && vStringChar (line, last) != ':') last--;
-		while (last && (isspace (vStringChar (line, last-1)))) last--;
+		while (last && (isspace ((int) vStringChar (line, last-1)))) last--;
 		while (first < last &&
-		       (isspace (vStringChar (line, first)) ||
+		       (isspace ((int) vStringChar (line, first)) ||
 		        vStringChar (line, first) == '-'))
 		    first++;
 		/* If there's anything left it is a fragment title */
 		if (first < last - 1)
 		{
 		    vStringChar (line, last) = 0;
-		    if (stricmp ("LIB", vStringValue (line) + first) &&
-		        stricmp ("PROGRAM", vStringValue (line) + first))
+		    if (strcasecmp ("LIB", vStringValue (line) + first) &&
+		        strcasecmp ("PROGRAM", vStringValue (line) + first))
 		    {
 			makeBetaTag (vStringValue (line) + first, K_FRAGMENT);
 		    }
@@ -186,30 +185,29 @@ static void findBetaTags (void)
 		    char c2;
 		    pos += 2; /* skip past << */
 		    /* skip past space before SLOT */
-		    while (pos < len &&
-		           isspace (vStringChar (line, pos)))
+		    while (pos < len && isspace ((int) vStringChar (line, pos)))
 		        pos++;
 		    /* skip past SLOT */
 		    if (pos+4 <= len &&
-		        !strnicmp (vStringValue(line) + pos, "SLOT", (size_t)4))
+		        !strncasecmp (vStringValue(line) + pos, "SLOT", (size_t)4))
 		        pos += 4;
 		    /* skip past space after SLOT */
 		    while (pos < len &&
-		           isspace (vStringChar (line, pos)))
+		           isspace ((int) vStringChar (line, pos)))
 		        pos++;
 		    eoname = pos;
 		    /* skip to end of name */
 		    while (eoname < len &&
 		           (c2 = vStringChar (line, eoname)) != '>' &&
 		           c2 != ':' &&
-			   !isspace (c2))
+			   !isspace ((int) c2))
 		    	eoname++;
 		    if (eoname < len)
 		    {
 		        vStringChar (line, eoname) = 0;
-			if (stricmp ("LIB", vStringValue (line) + pos) &&
-			    stricmp ("PROGRAM", vStringValue (line) + pos) &&
-			    stricmp ("SLOT", vStringValue (line) + pos))
+			if (strcasecmp ("LIB", vStringValue (line) + pos) &&
+			    strcasecmp ("PROGRAM", vStringValue (line) + pos) &&
+			    strcasecmp ("SLOT", vStringValue (line) + pos))
 			{
 			    makeBetaTag (vStringValue (line) + pos, K_SLOT);
 			}
@@ -239,7 +237,7 @@ static void findBetaTags (void)
 		    /* Found pattern name, get start and end */
 		    int eoname = pos;
 		    int soname;
-		    while (eoname && isspace (vStringChar (line, eoname-1)))
+		    while (eoname && isspace ((int) vStringChar (line, eoname-1)))
 			eoname--;
 		foundanothername:
 		    /* terminate right after name */
@@ -255,7 +253,7 @@ static void findBetaTags (void)
 			makeBetaTag (vStringValue (line) + soname, K_PATTERN);
 			/* scan back past white space */
 			while (soname &&
-				isspace (vStringChar (line, soname-1)))
+				isspace ((int) vStringChar (line, soname-1)))
 			    soname--;
 			if (soname && vStringChar (line, soname-1) == ',')
 			{
