@@ -1,7 +1,7 @@
 /*
-*   $Id: c.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: c.c,v 1.5 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -22,10 +22,10 @@
 #include "entry.h"
 #include "get.h"
 #include "keyword.h"
-#include "main.h"
 #include "options.h"
 #include "parse.h"
 #include "read.h"
+#include "routines.h"
 
 /*
 *   MACROS
@@ -37,8 +37,8 @@
 #define parentDecl(st)		((st)->parent == NULL ? \
 				DECL_NONE : (st)->parent->declaration)
 #define isType(token,t)		(boolean) ((token)->type == (t))
-#define insideEnumBody(st)	(boolean) ((st)->parent == NULL ? FALSE : \
-				    ((st)->parent->declaration == DECL_ENUM))
+#define insideEnumBody(st)	((st)->parent == NULL ? FALSE : \
+			    (boolean) ((st)->parent->declaration == DECL_ENUM))
 #define isExternCDecl(st,c)	(boolean) ((c) == STRING_SYMBOL  && \
 		    ! (st)->haveQualifyingName  && (st)->scope == SCOPE_EXTERN)
 
@@ -1783,8 +1783,12 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 			token->type = TOKEN_PAREN_NAME;
 		    else if (isType (token, TOKEN_KEYWORD))
 		    {
-			info->isKnrParamList = FALSE;
-			info->isNameCandidate = FALSE;
+			if (token->keyword != KEYWORD_CONST &&
+			    token->keyword != KEYWORD_VOLATILE)
+			{
+			    info->isKnrParamList = FALSE;
+			    info->isNameCandidate = FALSE;
+			}
 		    }
 		}
 		else
@@ -2002,27 +2006,26 @@ static void parseGeneralToken (statementInfo *const st, const int c)
  */
 static void nextToken (statementInfo *const st)
 {
-    tokenInfo *token = activeToken (st);
-
+    tokenInfo *token;
     do
     {
 	int c = skipToNonWhite ();
-
 	switch (c)
 	{
-	    case EOF: longjmp (Exception, (int) ExceptionEOF);		break;
-	    case '(': analyzeParens (st);  token = activeToken (st);	break;
-	    case '*': st->haveQualifyingName = FALSE;			break;
-	    case ',': setToken (st, TOKEN_COMMA);			break;
-	    case ':': processColon (st);					break;
-	    case ';': setToken (st, TOKEN_SEMICOLON);			break;
-	    case '<': skipToMatch ("<>");				break;
-	    case '=': processInitializer (st);				break;
-	    case '[': skipToMatch ("[]");				break;
-	    case '{': setToken (st, TOKEN_BRACE_OPEN);			break;
-	    case '}': setToken (st, TOKEN_BRACE_CLOSE);			break;
-	    default:  parseGeneralToken (st, c);				break;
+	    case EOF: longjmp (Exception, (int) ExceptionEOF);	break;
+	    case '(': analyzeParens (st);			break;
+	    case '*': st->haveQualifyingName = FALSE;		break;
+	    case ',': setToken (st, TOKEN_COMMA);		break;
+	    case ':': processColon (st);			break;
+	    case ';': setToken (st, TOKEN_SEMICOLON);		break;
+	    case '<': skipToMatch ("<>");			break;
+	    case '=': processInitializer (st);			break;
+	    case '[': skipToMatch ("[]");			break;
+	    case '{': setToken (st, TOKEN_BRACE_OPEN);		break;
+	    case '}': setToken (st, TOKEN_BRACE_CLOSE);		break;
+	    default:  parseGeneralToken (st, c);		break;
 	}
+	token = activeToken (st);
     } while (isType (token, TOKEN_NONE));
 }
 
@@ -2175,7 +2178,8 @@ static void tagCheck (statementInfo *const st)
 	    {
 		if (isContextualKeyword (prev2))
 		    st->scope = SCOPE_EXTERN;
-		qualifyVariableTag (st, prev);
+		else
+		    qualifyVariableTag (st, prev);
 	    }
 	    else if (isType (prev, TOKEN_ARGS)  &&  isType (prev2, TOKEN_NAME))
 	    {
