@@ -1,7 +1,7 @@
 /*
-*   $Id: ctags.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: ctags.h,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -15,12 +15,12 @@
 *   MACROS
 */
 #ifndef PROGRAM_VERSION
-# define PROGRAM_VERSION "5.0.1"
+# define PROGRAM_VERSION "5.2.3"
 #endif
 #define PROGRAM_NAME	"Exuberant Ctags"
 #define PROGRAM_URL	"http://ctags.sourceforge.net"
 #define AUTHOR_NAME	"Darren Hiebert"
-#define AUTHOR_EMAIL	"darren@hiebert.com"
+#define AUTHOR_EMAIL	"dhiebert@users.sourceforge.net"
 
 #endif	/* _CTAGS_H */
 
