@@ -1,7 +1,7 @@
 /*
-*   $Id: eiffel.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: eiffel.c,v 1.12 2002/02/23 05:32:20 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -15,23 +15,29 @@
 */
 #include "general.h"	/* must always come first */
 
+#ifdef TYPE_REFERENCE_TOOL
+#include <stdio.h>
+#endif
 #include <string.h>
 #include <limits.h>
 #include <ctype.h>	/* to define tolower () */
 #include <setjmp.h>
 
 #include "debug.h"
-#include "entry.h"
 #include "keyword.h"
-#include "main.h"
+#include "routines.h"
+#include "vstring.h"
+#ifndef TYPE_REFERENCE_TOOL
+#include "entry.h"
 #include "options.h"
 #include "parse.h"
 #include "read.h"
-#include "vstring.h"
+#endif
 
 /*
 *   MACROS
 */
+#define isident(c)		(isalnum(c) || (c) == '_')
 #define isFreeOperatorChar(c)	((c) == '@' || (c) == '#' || \
 				 (c) == '|' || (c) == '&')
 #define isType(token,t)		(boolean) ((token)->type == (t))
@@ -78,8 +84,9 @@ typedef enum eTokenType {
     TOKEN_CLOSE_BRACE,
     TOKEN_CLOSE_BRACKET,
     TOKEN_CLOSE_PAREN,
-    TOKEN_COMMA,
     TOKEN_COLON,
+    TOKEN_COMMA,
+    TOKEN_CONSTRAINT,
     TOKEN_DOT,
     TOKEN_DOLLAR,
     TOKEN_IDENTIFIER,
@@ -110,6 +117,19 @@ typedef struct sTokenInfo {
 
 static langType Lang_eiffel;
 
+#ifdef TYPE_REFERENCE_TOOL
+
+static const char *FileName;
+static FILE *File;
+static int PrintClass;
+static int PrintReferences;
+static int SelfReferences;
+static int Debug;
+static stringList *GenericNames;
+static stringList *ReferencedTypes;
+
+#else
+
 typedef enum {
     EKIND_CLASS, EKIND_FEATURE, EKIND_LOCAL, EKIND_QUALIFIED_TAGS
 } eiffelKind;
@@ -120,6 +140,10 @@ static kindOption EiffelKinds [] = {
     { FALSE, 'l', "local",   "local entities"}
 };
 
+#endif
+
+static langType Lang_eiffel;
+
 static jmp_buf Exception;
 
 static const keywordDesc EiffelKeywordTable [] = {
@@ -199,6 +223,62 @@ static void buildEiffelKeywordHash (void)
     }
 }
 
+#ifdef TYPE_REFERENCE_TOOL
+
+static void addGenericName (tokenInfo *const token)
+{
+    if (vStringLength (token->string) > 0)
+	stringListAdd (GenericNames, vStringNewCopy (token->string));
+}
+
+static boolean isGeneric (tokenInfo *const token)
+{
+    return (boolean) stringListHasInsensitive (
+	GenericNames, vStringValue (token->string));
+}
+
+static void reportType (tokenInfo *const token)
+{
+    if (vStringLength (token->string) > 0  && ! isGeneric (token)  &&
+	(SelfReferences || strcasecmp (vStringValue (
+	    token->string), vStringValue (token->className)) != 0) &&
+	! stringListHasInsensitive (
+	    ReferencedTypes, vStringValue (token->string)))
+    {
+	printf ("%s\n", vStringValue (token->string));
+	stringListAdd (ReferencedTypes, vStringNewCopy (token->string));
+    }
+}
+
+static int fileGetc (void)
+{
+    int c = getc (File);
+    if (c == '\r')
+    {
+	c = getc (File);
+	if (c != '\n')
+	{
+	    ungetc (c, File);
+	    c = '\n';
+	}
+    }
+    if (Debug > 0  &&  c != EOF)
+	putc (c, stderr);
+    return c;
+}
+
+static int fileUngetc (c)
+{
+    return ungetc (c, File);
+}
+
+extern char *readLine (vString *const vLine, FILE *const fp)
+{
+    return NULL;
+}
+
+#else
+
 /*
 *   Tag generation functions
 */
@@ -238,7 +318,7 @@ static void makeEiffelFeatureTag (tokenInfo *const token)
 
 	makeTagEntry (&e);
 
-	if (Option.qualifiedTags)
+	if (Option.include.qualifiedTags)
 	{
 	    vString* qualified = vStringNewInit (vStringValue (token->className));
 	    vStringPut (qualified, '.');
@@ -273,11 +353,12 @@ static void makeEiffelLocalTag (tokenInfo *const token)
 	e.extensionFields.scope [1] = vStringValue (scope);
 
 	makeTagEntry (&e);
-
 	vStringDelete (scope);
     }
 }
 
+#endif
+
 /*
 *   Parsing functions
 */
@@ -305,6 +386,8 @@ static vString *parseInteger (int c)
 	string = vStringNew ();
     vStringClear (string);
 
+    if (c == '\0')
+	c = fileGetc ();
     if (c == '-')
     {
 	vStringPut (string, c);
@@ -343,7 +426,7 @@ static vString *parseNumeric (int c)
 	vStringPut (string, c);
 	vStringCat (string, parseInteger ('\0'));
     }
-    else
+    else if (!isspace (c))
 	fileUngetc (c);
 
     vStringTerminate (string);
@@ -473,7 +556,10 @@ static void parseString (vString *const string)
 	{
 	    vStringPut (string, c);
 	    if (verbatim)
+	    {
 		vStringPut (lastLine, c);
+		vStringTerminate (lastLine);
+	    }
 	    prev = c;
 	}
     }
@@ -493,7 +579,8 @@ static void parseIdentifier (vString *const string, const int firstChar)
     } while (isident (c));
 
     vStringTerminate (string);
-    fileUngetc (c);		/* unget non-identifier character */
+    if (!isspace (c))
+	fileUngetc (c);		/* unget non-identifier character */
 }
 
 static void parseFreeOperator (vString *const string, const int firstChar)
@@ -507,7 +594,8 @@ static void parseFreeOperator (vString *const string, const int firstChar)
     } while (c > ' ');
 
     vStringTerminate (string);
-    fileUngetc (c);		/* unget non-identifier character */
+    if (!isspace (c))
+	fileUngetc (c);		/* unget non-identifier character */
 }
 
 static keywordId analyzeToken (vString *const name)
@@ -561,13 +649,19 @@ getNextChar:
 
 	case '-':
 	    c = fileGetc ();
-	    if (c != '-')		/* is this the start of a comment? */
-		fileUngetc (c);
-	    else
+	    if (c == '>')
+		token->type = TOKEN_CONSTRAINT;
+	    else if (c == '-')		/* is this the start of a comment? */
 	    {
 		skipToCharacter ('\n');
 		goto getNextChar;
 	    }
+	    else
+	    {
+		if (!isspace (c))
+		    fileUngetc (c);
+ 		token->type = TOKEN_OPERATOR;
+	    }
 	    break;
 
 	case '?':
@@ -578,34 +672,35 @@ getNextChar:
 	    else
 	    {
 		token->type = TOKEN_COLON;
-		fileUngetc (c);
+		if (!isspace (c))
+		    fileUngetc (c);
 	    }
 	    break;
 
 	case '<':
 	    c = fileGetc ();
-	    if (c != '='  &&  c != '>')
+	    if (c != '='  &&  c != '>'  &&  !isspace (c))
 		fileUngetc (c);
 	    token->type = TOKEN_OPERATOR;
 	    break;
 
 	case '>':
 	    c = fileGetc ();
-	    if (c != '='  &&  c != '>')
+	    if (c != '='  &&  c != '>'  &&  !isspace (c))
 		fileUngetc (c);
 	    token->type = TOKEN_OPERATOR;
 	    break;
 
 	case '/':
 	    c = fileGetc ();
-	    if (c != '/'  &&  c != '=')
+	    if (c != '/'  &&  c != '='  &&  !isspace (c))
 		fileUngetc (c);
 	    token->type = TOKEN_OPERATOR;
 	    break;
 
 	case '\\':
 	    c = fileGetc ();
-	    if (c != '\\')
+	    if (c != '\\'  &&  !isspace (c))
 		fileUngetc (c);
 	    token->type = TOKEN_OPERATOR;
 	    break;
@@ -657,36 +752,89 @@ static boolean isIdentifierMatch (const tokenInfo *const token,
 				  const char *const name)
 {
     return (boolean) (isType (token, TOKEN_IDENTIFIER)  &&
-		     stricmp (vStringValue (token->string), name) == 0);
+		     strcasecmp (vStringValue (token->string), name) == 0);
 }
 
-static void locateToken (tokenInfo *const token, const tokenType type)
+static void findToken (tokenInfo *const token, const tokenType type)
 {
-    do
+    while (! isType (token, type))
 	readToken (token);
-    while (token->type != type);
-
-    readToken (token);
 }
 
-static void skipPastMatchingBracket (tokenInfo *const token)
+static void findKeyword (tokenInfo *const token, const keywordId keyword)
 {
-    int depth = 0;
+    while (! isKeyword (token, keyword))
+	readToken (token);
+}
 
+static void parseGeneric (tokenInfo *const token, boolean __unused__ declaration)
+{
+    unsigned int depth = 0;
+#ifdef TYPE_REFERENCE_TOOL
+    boolean constraint = FALSE;
+#endif
+    Assert (isType (token, TOKEN_OPEN_BRACKET));
     do
     {
 	if (isType (token, TOKEN_OPEN_BRACKET))
 	    ++depth;
 	else if (isType (token, TOKEN_CLOSE_BRACKET))
 	    --depth;
+#ifdef TYPE_REFERENCE_TOOL
+	else if (declaration)
+	{
+	    if (depth == 1)
+	    {
+		if (isType (token, TOKEN_CONSTRAINT))
+		    constraint = TRUE;
+		else if (isKeyword (token, KEYWORD_create))
+		    findKeyword (token, KEYWORD_end);
+		else if (isType (token, TOKEN_IDENTIFIER))
+		{
+		    if (constraint)
+			reportType (token);
+		    else
+			addGenericName (token);
+		    constraint = FALSE;
+		}
+	    }
+	    else if (isKeyword (token, KEYWORD_like))
+		readToken (token);
+	    else if (isType (token, TOKEN_IDENTIFIER))
+		reportType (token);
+	}
+	else
+	{
+	    if (isType (token, TOKEN_OPEN_BRACKET))
+		++depth;
+	    else if (isType (token, TOKEN_IDENTIFIER))
+		reportType (token);
+	    else if (isKeyword (token, KEYWORD_like))
+		readToken (token);
+	}
+#endif
 	readToken (token);
     } while (depth > 0);
 }
 
-static void skipEntityType (tokenInfo *const token)
+static void parseType (tokenInfo *const token)
 {
-    /*  When called, we are at the ':'. Move to the type.
-     */
+    boolean bitType;
+    Assert (isType (token, TOKEN_IDENTIFIER));
+#ifdef TYPE_REFERENCE_TOOL
+    reportType (token);
+#endif
+    bitType = (boolean)(strcmp ("BIT", vStringValue (token->string)) == 0);
+    readToken (token);
+    if (bitType && isType (token, TOKEN_NUMERIC))
+	readToken (token);
+    else if (isType (token, TOKEN_OPEN_BRACKET))
+	parseGeneric (token, FALSE);
+}
+
+static void parseEntityType (tokenInfo *const token)
+{
+    Assert (isType (token, TOKEN_COLON));
     readToken (token);
 
     if (isKeyword (token, KEYWORD_expanded))
@@ -694,31 +842,40 @@ static void skipEntityType (tokenInfo *const token)
 
     /*  Skip over the type name, with possible generic parameters.
      */
-    if (isKeyword (token, KEYWORD_like)  ||
-	stricmp (vStringValue (token->string), "BIT") == 0)
+    if (isType (token, TOKEN_IDENTIFIER))
+	parseType (token);
+    else if (isKeyword (token, KEYWORD_like))
     {
 	readToken (token);
+	if (isType (token, TOKEN_IDENTIFIER) ||
+		isKeyword (token, KEYWORD_Current))
+	    readToken (token);
     }
-    readToken (token);
-    if (isType (token, TOKEN_OPEN_BRACKET))
-	skipPastMatchingBracket (token);
 }
 
+
 static void parseLocal (tokenInfo *const token)
 {
-    readToken (token);	    /* skip past "local" keyword */
+    Assert (isKeyword (token, KEYWORD_local));
+    readToken (token);
 
     /*  Check keyword first in case local clause is empty
      */
     while (! isKeyword (token, KEYWORD_do)  &&
 	   ! isKeyword (token, KEYWORD_once))
     {
+#ifndef TYPE_REFERENCE_TOOL
 	if (isType (token, TOKEN_IDENTIFIER))
 	    makeEiffelLocalTag (token);
 
+#endif
 	readToken (token);
 	if (isType (token, TOKEN_COLON))
-	    skipEntityType (token);
+	{
+	    readToken (token);
+	    if (isType (token, TOKEN_IDENTIFIER))
+		parseType (token);
+	}
     }
 }
 
@@ -729,6 +886,9 @@ static void findFeatureEnd (tokenInfo *const token)
     switch (token->keyword)
     {
 	default:
+	    if (isType (token, TOKEN_OPERATOR)) /* sign of manifest constant */
+		readToken (token);
+	    readToken (token);		/* skip to next token after constant */
 	    break;
 
 	case KEYWORD_deferred:
@@ -743,6 +903,25 @@ static void findFeatureEnd (tokenInfo *const token)
 
 	    while (depth > 0)
 	    {
+#ifdef TYPE_REFERENCE_TOOL
+		if (isType (token, TOKEN_OPEN_BRACE))
+		{
+		    readToken (token);
+		    if (isType (token, TOKEN_IDENTIFIER))
+			parseType (token);
+		    if (isType (token, TOKEN_CLOSE_BRACE))
+			readToken (token);
+		}
+		else if (isType (token, TOKEN_BANG))
+		{
+		    readToken (token);
+		    if (isType (token, TOKEN_IDENTIFIER))
+			parseType (token);
+		    if (isType (token, TOKEN_BANG))
+			readToken (token);
+		}
+		else
+#endif
 		switch (token->keyword)
 		{
 		    case KEYWORD_check:
@@ -775,6 +954,8 @@ static boolean readFeatureName (tokenInfo *const token)
 {
     boolean isFeatureName = FALSE;
 
+    if (isKeyword (token, KEYWORD_frozen))
+	readToken (token);
     if (isType (token, TOKEN_IDENTIFIER))
 	isFeatureName = TRUE;
     else if (isKeyword (token, KEYWORD_infix)  ||
@@ -787,67 +968,93 @@ static boolean readFeatureName (tokenInfo *const token)
     return isFeatureName;
 }
 
-static void parseFeature (tokenInfo *const token)
+static void parseArguments (tokenInfo *const token)
 {
+#ifndef TYPE_REFERENCE_TOOL
+    findToken (token, TOKEN_CLOSE_PAREN);
+    readToken (token);
+#else
+    Assert (isType (token, TOKEN_OPEN_PAREN));
+    readToken (token);
     do
     {
-	if (readFeatureName (token))
-	    makeEiffelFeatureTag (token);
+	if (! isType (token, TOKEN_COLON))
+	    readToken (token);
+	else
+	{
+	    readToken (token);
+	    if (isType (token, TOKEN_IDENTIFIER))
+		parseType (token);
+	}
+    } while (! isType (token, TOKEN_CLOSE_PAREN));
+    readToken (token);
+#endif
+}
+
+static boolean parseFeature (tokenInfo *const token)
+{
+    boolean found = FALSE;
+    while (readFeatureName (token))
+    {
+	found = TRUE;
+#ifndef TYPE_REFERENCE_TOOL
+	makeEiffelFeatureTag (token);
+#endif
 	readToken (token);
-    } while (isType (token, TOKEN_COMMA));
-
-
-    /*  Skip signature.
-     */
-    if (isType (token, TOKEN_OPEN_PAREN))	/* arguments? */
-	locateToken (token, TOKEN_CLOSE_PAREN);
-    if (isType (token, TOKEN_COLON))		/* a query? */
-	skipEntityType (token);
-
-    if (isKeyword (token, KEYWORD_is))
-	findFeatureEnd (token);
+	if (isType (token, TOKEN_COMMA))
+	    readToken (token);
+    }
+    if (found)
+    {
+	if (isType (token, TOKEN_OPEN_PAREN))	/* arguments? */
+	    parseArguments (token);
+	if (isType (token, TOKEN_COLON))	/* a query? */
+	    parseEntityType (token);
+	if (isKeyword (token, KEYWORD_obsolete))
+	{
+	    readToken (token);
+	    if (isType (token, TOKEN_STRING))
+		readToken (token);
+	}
+	if (isKeyword (token, KEYWORD_is))
+	    findFeatureEnd (token);
+    }
+    return found;
 }
 
 static void parseExport (tokenInfo *const token)
 {
     token->isExported = TRUE;
-
     readToken (token);
-    if (isType (token, TOKEN_OPEN_BRACE)) do
+    if (isType (token, TOKEN_OPEN_BRACE))
     {
-	readToken (token);
-	if (isType (token, TOKEN_CLOSE_BRACE))
+	token->isExported = FALSE;
+	while (! isType (token, TOKEN_CLOSE_BRACE))
 	{
-	    token->isExported = FALSE;
+	    if (isType (token, TOKEN_IDENTIFIER))
+		token->isExported |= !isIdentifierMatch (token, "NONE");
 	    readToken (token);
 	}
-	else
-	{
-	    if (isIdentifierMatch (token, "NONE"))
-		token->isExported = FALSE;
-	    locateToken (token, TOKEN_CLOSE_BRACE);
-	}
-    } while (isType (token, TOKEN_COMMA));
-}
-
-static void locateKeyword (tokenInfo *const token, const keywordId keyword)
-{
-    do
 	readToken (token);
-    while (! isKeyword (token, keyword));
-
-    readToken (token);
+    }
 }
 
 static void parseFeatureClauses (tokenInfo *const token)
 {
+    Assert (isKeyword (token, KEYWORD_feature));
     do
     {
-	while (isKeyword (token, KEYWORD_feature))
+	if (isKeyword (token, KEYWORD_feature))
 	    parseExport (token);
-	parseFeature (token);
+	if (! isKeyword (token, KEYWORD_invariant) &&
+	    ! isKeyword (token, KEYWORD_indexing))
+	{
+	    if (! parseFeature (token))
+		readToken (token);
+	}
     } while (! isKeyword (token, KEYWORD_end) &&
-	     ! isKeyword (token, KEYWORD_invariant));
+	     ! isKeyword (token, KEYWORD_invariant) &&
+	     ! isKeyword (token, KEYWORD_indexing));
 }
 
 static void parseRename (tokenInfo *const token)
@@ -862,22 +1069,48 @@ static void parseRename (tokenInfo *const token)
 		readToken (token);
 		if (readFeatureName (token))
 		{
+#ifndef TYPE_REFERENCE_TOOL
 		    makeEiffelFeatureTag (token);	/* renamed feature */
+#endif
 		    readToken (token);
 		}
 	    }
 	}
     } while (isType (token, TOKEN_COMMA));
 
-    if (! isKeyword (token, KEYWORD_end))
-	locateKeyword (token, KEYWORD_end);
+    findKeyword (token, KEYWORD_end);
+    readToken (token);
 }
 
+
 static void parseInherit (tokenInfo *const token)
 {
-    do
+    Assert (isKeyword (token, KEYWORD_inherit));
+#ifdef TYPE_REFERENCE_TOOL
+    readToken (token);
+    while (isType (token, TOKEN_IDENTIFIER))
     {
-	readToken (token);
+	parseType (token);
+	if (isType (token, TOKEN_KEYWORD))
+	{
+	    switch (token->keyword)	/* check for feature adaptation */
+	    {
+		case KEYWORD_rename:
+		case KEYWORD_export:
+		case KEYWORD_undefine:
+		case KEYWORD_redefine:
+		case KEYWORD_select:
+		    findKeyword (token, KEYWORD_end);
+		    readToken (token);
+		default: break;
+	    }
+	}
+    }
+#else
+    readToken (token);
+    while (isType (token, TOKEN_IDENTIFIER))
+    {
+	parseType (token);
 	switch (token->keyword)	    /* check for feature adaptation */
 	{
 	    case KEYWORD_rename:
@@ -890,7 +1123,8 @@ static void parseInherit (tokenInfo *const token)
 	    case KEYWORD_undefine:
 	    case KEYWORD_redefine:
 	    case KEYWORD_select:
-		locateKeyword (token, KEYWORD_end);
+		findKeyword (token, KEYWORD_end);
+		readToken (token);
 		break;
 
 	    case KEYWORD_end:
@@ -899,20 +1133,36 @@ static void parseInherit (tokenInfo *const token)
 
 	    default: break;
 	}
-    } while (! isType (token, TOKEN_KEYWORD));
+    }
+#endif
 }
 
 static void parseClass (tokenInfo *const token)
 {
+    Assert (isKeyword (token, KEYWORD_class));
+    readToken (token);
     if (isType (token, TOKEN_IDENTIFIER))
     {
+#ifndef TYPE_REFERENCE_TOOL
 	makeEiffelClassTag (token);
 	readToken (token);
+#else
+	vStringCopy (token->className, token->string);
+	if (PrintClass)
+	    puts (vStringValue (token->className));
+	if (! PrintReferences)
+	    exit (0);
+	readToken (token);
+#endif
     }
 
     do
     {
-	switch (token->keyword)
+	if (isType (token, TOKEN_OPEN_BRACKET))
+	    parseGeneric (token, TRUE);
+	else if (! isType (token, TOKEN_KEYWORD))
+	    readToken (token);
+	else switch (token->keyword)
 	{
 	    case KEYWORD_inherit:  parseInherit (token);        break;
 	    case KEYWORD_feature:  parseFeatureClauses (token); break;
@@ -959,12 +1209,14 @@ static void findEiffelTags (void)
     exception = (exception_t) (setjmp (Exception));
     while (exception == ExceptionNone)
     {
-	locateKeyword (token, KEYWORD_class);
+	findKeyword (token, KEYWORD_class);
 	parseClass (token);
     }
     deleteToken (token);
 }
 
+#ifndef TYPE_REFERENCE_TOOL
+
 extern parserDefinition* EiffelParser (void)
 {
     static const char *const extensions [] = { "e", NULL };
@@ -977,4 +1229,88 @@ extern parserDefinition* EiffelParser (void)
     return def;
 }
 
+#else
+
+static void findReferences (void)
+{
+    ReferencedTypes = stringListNew ();
+    GenericNames = stringListNew ();
+    initialize (0);
+
+    findEiffelTags ();
+
+    stringListDelete (GenericNames);
+    GenericNames = NULL;
+    stringListDelete (ReferencedTypes);
+    ReferencedTypes = NULL;
+}
+
+static const char *const Usage =
+    "Prints names of types referenced by an Eiffel language file.\n"
+    "\n"
+    "Usage: %s [-cdrs] [file_name | -]\n"
+    "\n"
+    "Options:\n"
+    "    -c    Print class name of current file (on first line of output).\n"
+    "    -d    Enable debug output.\n"
+    "    -r    Print types referenced by current file (default unless -c).\n"
+    "    -s    Include self-references.\n"
+    "\n";
+
+extern main (int argc, char** argv)
+{
+    int i;
+    for (i = 1  ;  argv [i] != NULL  ;  ++i)
+    {
+	const char *const arg = argv [i];
+	if (arg [0] == '-')
+	{
+	    int j;
+	    if (arg [1] == '\0')
+	    {
+		    File = stdin;
+		    FileName = "stdin";
+	    }
+	    else for (j = 1  ;  arg [j] != '\0'  ;  ++j) switch (arg [j])
+	    {
+		case 'c':  PrintClass      = 1; break;
+		case 'r':  PrintReferences = 1; break;
+		case 's':  SelfReferences  = 1; break;
+		case 'd':  Debug           = 1; break;
+		default:
+		    fprintf (stderr, "%s: unknown option: %c\n", argv [0], arg [1]);
+		    fprintf (stderr, Usage, argv [0]);
+		    exit (1);
+		    break;
+	    }
+	}
+	else if (File != NULL)
+	{
+	    fprintf (stderr, Usage, argv [0]);
+	    exit (1);
+	}
+	else
+	{
+	    FileName = arg;
+	    File = fopen (FileName, "r");
+	    if (File == NULL)
+	    {
+		perror (argv [0]);
+		exit (1);
+	    }
+	}
+    }
+    if (! PrintClass)
+	PrintReferences = 1;
+    if (File == NULL)
+    {
+	fprintf (stderr, Usage, argv [0]);
+	exit (1);
+    }
+
+    findReferences ();
+}
+
+#endif
+
 /* vi:set tabstop=8 shiftwidth=4: */
