@@ -1,7 +1,7 @@
 /*
-*   $Id: entry.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: entry.c,v 1.5 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -44,6 +44,7 @@
 #include "main.h"
 #include "options.h"
 #include "read.h"
+#include "routines.h"
 #include "sort.h"
 #include "strlist.h"
 
@@ -202,27 +203,27 @@ static void updateSortedFlag (const char *const line,
  */
 static long unsigned int updatePseudoTags (FILE *const fp)
 {
-    enum { maxClassLength = 20 };
-    char class [maxClassLength + 1];
+    enum { maxEntryLength = 20 };
+    char entry [maxEntryLength + 1];
     unsigned long linesRead = 0;
     fpos_t startOfLine;
-    size_t classLength;
+    size_t entryLength;
     const char *line;
 
-    sprintf (class, "%sTAG_FILE", PSEUDO_TAG_PREFIX);
-    classLength = strlen (class);
-    Assert (classLength < maxClassLength);
+    sprintf (entry, "%sTAG_FILE", PSEUDO_TAG_PREFIX);
+    entryLength = strlen (entry);
+    Assert (entryLength < maxEntryLength);
 
     fgetpos (fp, &startOfLine);
     line = readLine (TagFile.vLine, fp);
-    while (line != NULL  &&  line [0] == class [0])
+    while (line != NULL  &&  line [0] == entry [0])
     {
 	++linesRead;
-	if (strncmp (line, class, classLength) == 0)
+	if (strncmp (line, entry, entryLength) == 0)
 	{
 	    char tab, classType [16];
 
-	    if (sscanf (line + classLength, "%15s%c", classType, &tab) == 2  &&
+	    if (sscanf (line + entryLength, "%15s%c", classType, &tab) == 2  &&
 		tab == '\t')
 	    {
 		if (strcmp (classType, "_SORTED") == 0)
@@ -337,7 +338,7 @@ extern void copyBytes (FILE* const fromFp, FILE* const toFp, const long size)
     do
     {
 	toRead = (0 < remaining && remaining < BufferSize) ?
-		    remaining : BufferSize;
+		    remaining : (long) BufferSize;
 	numRead = fread (buffer, (size_t) 1, (size_t) toRead, fromFp);
 	if (fwrite (buffer, (size_t)1, (size_t)numRead, toFp) < (size_t)numRead)
 	    error (FATAL | PERROR, "cannot complete write");
@@ -691,46 +692,52 @@ static int writeEtagsEntry (const tagEntryInfo *const tag)
 static int addExtensionFields (const tagEntryInfo *const tag)
 {
     const char* const kindKey = Option.extensionFields.kindKey ? "kind:" : "";
-    int length = fprintf (TagFile.fp, ";\"");
+    boolean first = TRUE;
+    const char* separator = ";\"";
+    const char* const empty = "";
+    int length = 0;
+/* "sep" returns a value only the first time it is evaluated */
+#define sep (first ? (first = FALSE, separator) : empty)
 
-    if (tag->kindName != NULL && (
-	 Option.extensionFields.kindLong  ||  tag->kind == '\0'))
-	length += fprintf (TagFile.fp, "\t%s%s", kindKey, tag->kindName);
-    else if (tag->kind != '\0'  && (
-	 ! Option.extensionFields.kindLong  ||  tag->kindName == NULL))
-	length += fprintf (TagFile.fp, "\t%s%c", kindKey, tag->kind);
+    if (tag->kindName != NULL && (Option.extensionFields.kindLong  ||
+	 (Option.extensionFields.kind  && tag->kind == '\0')))
+	length += fprintf (TagFile.fp,"%s\t%s%s", sep, kindKey, tag->kindName);
+    else if (tag->kind != '\0'  && (Option.extensionFields.kind  ||
+	    (Option.extensionFields.kindLong  &&  tag->kindName == NULL)))
+	length += fprintf (TagFile.fp, "%s\t%s%c", sep, kindKey, tag->kind);
 
     if (Option.extensionFields.lineNumber)
-	length += fprintf (TagFile.fp, "\tline:%ld", tag->lineNumber);
+	length += fprintf (TagFile.fp, "%s\tline:%ld", sep, tag->lineNumber);
 
     if (Option.extensionFields.language  &&  tag->language != NULL)
-	length += fprintf (TagFile.fp, "\tlanguage:%s", tag->language);
+	length += fprintf (TagFile.fp, "%s\tlanguage:%s", sep, tag->language);
 
     if (Option.extensionFields.scope  &&
 	    tag->extensionFields.scope [0] != NULL  &&
 	    tag->extensionFields.scope [1] != NULL)
-	length += fprintf (TagFile.fp, "\t%s:%s",
+	length += fprintf (TagFile.fp, "%s\t%s:%s", sep,
 			   tag->extensionFields.scope [0],
 			   tag->extensionFields.scope [1]);
 
     if (Option.extensionFields.fileScope  &&  tag->isFileScope)
-	length += fprintf (TagFile.fp, "\tfile:");
+	length += fprintf (TagFile.fp, "%s\tfile:", sep);
 
     if (Option.extensionFields.inheritance  &&
 	    tag->extensionFields.inheritance != NULL)
-	length += fprintf (TagFile.fp, "\tinherits:%s",
+	length += fprintf (TagFile.fp, "%s\tinherits:%s", sep,
 			   tag->extensionFields.inheritance);
 
     if (Option.extensionFields.access  &&  tag->extensionFields.access != NULL)
-	length += fprintf (TagFile.fp, "\taccess:%s",
+	length += fprintf (TagFile.fp, "%s\taccess:%s", sep,
 			   tag->extensionFields.access);
 
     if (Option.extensionFields.implementation  &&
 	    tag->extensionFields.implementation != NULL)
-	length += fprintf (TagFile.fp, "\timplementation:%s",
+	length += fprintf (TagFile.fp, "%s\timplementation:%s", sep,
 			   tag->extensionFields.implementation);
 
     return length;
+#undef sep
 }
 
 static int writePatternEntry (const tagEntryInfo *const tag)
@@ -797,6 +804,7 @@ extern void makeTagEntry (const tagEntryInfo *const tag)
 
 	++TagFile.numTags.added;
 	rememberMaxLengths (strlen (tag->name), (size_t) length);
+	DebugStatement ( fflush (TagFile.fp); )
     }
 }
 
