@@ -1,7 +1,7 @@
 /*
-*   $Id: fortran.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: fortran.c,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -23,15 +23,16 @@
 #include "debug.h"
 #include "entry.h"
 #include "keyword.h"
-#include "main.h"
 #include "options.h"
 #include "parse.h"
 #include "read.h"
+#include "routines.h"
 #include "vstring.h"
 
 /*
 *   MACROS
 */
+#define isident(c)		(isalnum(c) || (c) == '_')
 #define isBlank(c)		(boolean) (c == ' ' || c == '\t')
 #define isType(token,t)		(boolean) ((token)->type == (t))
 #define isKeyword(token,k)	(boolean) ((token)->keyword == (k))
