@@ -1,7 +1,7 @@
 /*
-*   $Id: get.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: get.c,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -20,7 +20,6 @@
 #include "debug.h"
 #include "entry.h"
 #include "get.h"
-#include "main.h"
 #include "options.h"
 #include "read.h"
 #include "vstring.h"
@@ -50,19 +49,21 @@ typedef struct sConditionalInfo {
     boolean ignoring;		/* current ignore state */
 } conditionalInfo;
 
+enum eState {
+    DRCTV_NONE,			/* no known directive - ignore to end of line */
+    DRCTV_DEFINE,		/* "#define" encountered */
+    DRCTV_HASH,			/* initial '#' read; determine directive */
+    DRCTV_IF,			/* "#if" or "#ifdef" encountered */
+    DRCTV_UNDEF			/* "#undef" encountered */
+};
+
 /*  Defines the current state of the pre-processor.
  */
 typedef struct sCppState {
     int	    ungetch, ungetch2;	/* ungotten characters, if any */
     boolean resolveRequired;	/* must resolve if/else/elif/endif branch */
     struct sDirective {
-	enum eState {
-	    DRCTV_NONE,		/* no known directive - ignore to end of line */
-	    DRCTV_DEFINE,	/* "#define" encountered */
-	    DRCTV_HASH,		/* initial '#' read; determine directive */
-	    DRCTV_IF,		/* "#if" or "#ifdef" encountered */
-	    DRCTV_UNDEF		/* "#undef" encountered */
-	} state;
+	enum eState state;	/* current directive being processed */
 	boolean	accept;		/* is a directive syntatically permitted? */
 	vString * name;		/* macro name */
 	unsigned int nestLevel;	/* level 0 is not used */
