@@ -1,7 +1,7 @@
 /*
-*   $Id: get.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: get.h,v 1.2 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -18,6 +18,21 @@
 
 #include "ctags.h"	/* to define langType */
 
+/*
+*   MACROS
+*/
+/*  Is the character valid as a character of a C identifier?
+ */
+#ifdef VMS
+# define isident(c)	(isalnum(c) || (c) == '_' || (c) == '$')
+#else
+# define isident(c)	(isalnum(c) || (c) == '_')
+#endif
+
+/*  Is the character valid as the first character of a C identifier?
+ */
+#define isident1(c)	(isalpha(c) || (c) == '_' || (c) == '~')
+
 /*
 *   FUNCTION PROTOTYPES
 */
