@@ -1,7 +1,7 @@
 /*
-*   $Id: options.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: options.c,v 1.8 2002/02/25 04:45:45 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -25,6 +25,7 @@
 #define OPTION_WRITE
 #include "options.h"
 #include "parse.h"
+#include "routines.h"
 
 /*
 *   MACROS
@@ -69,7 +70,7 @@ enum eOptionLimits {
 
 typedef struct sOptionDescription {
     int usedByEtags;
-    const char *const description;
+    const char *description;
 } optionDescription;
 
 typedef void (*parametricOptionHandler) (const char *const option, const char *const parameter);
@@ -90,7 +91,9 @@ typedef const struct {
 *   DATA DEFINITIONS
 */
 
-static boolean ParsedLeadingOptions = FALSE;
+static boolean NonOptionEncountered = FALSE;
+
+static stringList *OptionFiles = NULL;
 
 static const char *const HeaderExtensions [] = {
     "h", "H", "hh", "hpp", "hxx", "h++", "inc", "def", NULL
@@ -138,7 +141,6 @@ optionValues Option = {
     TRUE,		/* --links */
     FALSE,		/* --filter */
     NULL,		/* --filter-terminator */
-    FALSE,		/* --qualified-tags */
     FALSE,		/* --tag-relative */
     FALSE,		/* --totals */
     FALSE,		/* --line-directives */
@@ -263,20 +265,21 @@ static optionDescription LongOptionDescription [] = {
  {1, NULL}
 };
 
-static const char* const License =
-"This program is free software; you can redistribute it and/or\n\
-modify it under the terms of the GNU General Public License\n\
-as published by the Free Software Foundation; either version 2\n\
-of the License, or (at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License\n\
-along with this program; if not, write to the Free Software\n\
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n";
+static const char* const License1 =
+"This program is free software; you can redistribute it and/or\n"
+"modify it under the terms of the GNU General Public License\n"
+"as published by the Free Software Foundation; either version 2\n"
+"of the License, or (at your option) any later version.\n"
+"\n";
+static const char* const License2 =
+"This program is distributed in the hope that it will be useful,\n"
+"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+"GNU General Public License for more details.\n"
+"\n"
+"You should have received a copy of the GNU General Public License\n"
+"along with this program; if not, write to the Free Software\n"
+"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n";
 
 /*  Contains a set of strings describing the set of "features" compiled into
  *  the code.
@@ -698,7 +701,7 @@ static void processLanguageForceOption (const char *const option,
 					const char *const parameter)
 {
     langType language;
-    if (stricmp (parameter, "auto") == 0)
+    if (strcasecmp (parameter, "auto") == 0)
 	language = LANG_AUTO;
     else
 	language = getNamedLanguage (parameter);
@@ -790,7 +793,7 @@ static char* processLanguageMap (char* map)
 	    else
 		clear = TRUE;
 	    for (p = list  ;  *p != ','  &&  *p != '\0'  ;  ++p) /*no-op*/ ;
-	    if (strnicmp (list, "default", p - list) == 0)
+	    if (strncasecmp (list, "default", p - list) == 0)
 	    {
 		verbose ("    Restoring default %s language map: ", getLanguageName (language));
 		installLanguageMapDefault (language);
@@ -1014,6 +1017,8 @@ static void readIgnoreList (const char *const list)
 static void addIgnoreListFromFile (const char *const fileName)
 {
     stringList* tokens = stringListNewFromFile (fileName);
+    if (tokens == NULL)
+	error (FATAL | PERROR, "cannot open \"%s\"", fileName);
     if (Option.ignore == NULL)
 	Option.ignore = tokens;
     else
@@ -1064,8 +1069,9 @@ static void printfFeatureList (void)
 
 static void printProgramIdentification (void)
 {
-    printf ("%s %s, Copyright (C) 1996-2001 %s\n",
+    printf ("%s %s, Copyright (C) 1996-2002 %s\n",
 	    PROGRAM_NAME, PROGRAM_VERSION, AUTHOR_NAME);
+    printf ("  Compiled: %s, %s\n", __DATE__, __TIME__);
     printf ("  Addresses: <%s>, %s\n", AUTHOR_EMAIL, PROGRAM_URL);
     printfFeatureList ();
 }
@@ -1141,7 +1147,7 @@ static void processFilterTerminatorOption (const char *const __unused__ option,
 static void processExtensionFieldsOption (const char *const option,
 					  const char *const parameter)
 {
-    struct sExtFields *const field = &Option.extensionFields;
+    struct sExtFields *field = &Option.extensionFields;
     const char *p = parameter;
     boolean mode = TRUE;
     int c;
@@ -1258,7 +1264,7 @@ static booleanOption BooleanOptions [] = {
 
 static void checkOptionOrder (const char* const option)
 {
-    if (ParsedLeadingOptions)
+    if (NonOptionEncountered)
 	error (FATAL, "-%s option may not follow a file name", option);
 }
 
@@ -1352,7 +1358,8 @@ static void processLongOption (const char *const option,
     {
 	printProgramIdentification ();
 	puts ("");
-	puts (License);
+	puts (License1);
+	puts (License2);
 	exit (0);
     }
     else if (isOption ("version"))
@@ -1500,21 +1507,40 @@ extern void parseOptions (cookedArgs* const args)
     while (! cArgOff (args)  &&  cArgIsOption (args))
 	parseOption (args);
     if (! cArgOff (args)  &&  ! cArgIsOption (args))
-	ParsedLeadingOptions = TRUE;
+	NonOptionEncountered = TRUE;
+}
+
+static const char *CheckFile;
+static boolean checkSameFile (const char *const fileName)
+{
+    return isSameFile (CheckFile, fileName);
 }
 
 static boolean parseFileOptions (const char* const fileName)
 {
     boolean fileFound = FALSE;
-    FILE* const fp = fopen (fileName, "r");
-    if (fp != NULL)
+    const char* const format = "Considering option file %s: %s\n";
+    CheckFile = fileName;
+    if (stringListHasTest (OptionFiles, checkSameFile))
+	verbose (format, fileName, "already read");
+    else
     {
-	cookedArgs* const args = cArgNewFromLineFile (fp);
-	verbose ("Reading options from %s\n", fileName);
-	parseOptions (args);
-	cArgDelete (args);
-	fclose (fp);
-	fileFound = TRUE;
+	FILE* const fp = fopen (fileName, "r");
+	if (fp == NULL)
+	    verbose (format, fileName, "not found");
+	else
+	{
+	    cookedArgs* const args = cArgNewFromLineFile (fp);
+	    vString* file = vStringNewInit (fileName);
+	    stringListAdd (OptionFiles, file);
+	    verbose (format, fileName, "reading...");
+	    parseOptions (args);
+	    if (NonOptionEncountered)
+		error (WARNING, "Ignoring non-option in %s\n", fileName);
+	    cArgDelete (args);
+	    fclose (fp);
+	    fileFound = TRUE;
+	}
     }
     return fileFound;
 }
@@ -1531,34 +1557,50 @@ extern void previewFirstOption (cookedArgs* const args)
 static void parseConfigurationFileOptions (void)
 {
     const char* const home = getenv ("HOME");
-    parseFileOptions ("/etc/ctags.conf");
-    parseFileOptions ("/usr/local/etc/ctags.conf");
+    const char* conf;
 #ifdef CUSTOM_CONFIGURATION_FILE
     parseFileOptions (CUSTOM_CONFIGURATION_FILE);
 #endif
+#ifdef MSDOS_STYLE_PATH
+    parseFileOptions ("/ctags.cnf");
+    conf = "ctags.cnf";
+#else
+    conf = ".ctags";
+#endif
+    parseFileOptions ("/etc/ctags.conf");
+    parseFileOptions ("/usr/local/etc/ctags.conf");
     if (home != NULL)
     {
-	vString* const dotFile = combinePathAndFile (home, ".ctags");
-	parseFileOptions (vStringValue (dotFile));
-	vStringDelete (dotFile);
+	vString* const homeFile = combinePathAndFile (home, conf);
+	parseFileOptions (vStringValue (homeFile));
+	vStringDelete (homeFile);
     }
-    parseFileOptions (".ctags");
+    parseFileOptions (conf);
 }
 
 static void parseEnvironmentOptions (void)
 {
     const char *envOptions = NULL;
+    const char* var = NULL;
 
     if (Option.etags)
-	envOptions = getenv (ETAGS_ENVIRONMENT);
+    {
+	var = ETAGS_ENVIRONMENT;
+	envOptions = getenv (var);
+    }
     if (envOptions == NULL)
-	envOptions = getenv (CTAGS_ENVIRONMENT);
+    {
+	var = CTAGS_ENVIRONMENT;
+	envOptions = getenv (var);
+    }
     if (envOptions != NULL  &&  envOptions [0] != '\0')
     {
 	cookedArgs* const args = cArgNewFromString (envOptions);
 	verbose ("Reading options from $CTAGS\n");
 	parseOptions (args);
 	cArgDelete (args);
+	if (NonOptionEncountered)
+	    error (WARNING, "Ignoring non-option in %s variable", var);
     }
 }
 
@@ -1574,10 +1616,19 @@ extern void readOptionConfiguration (void)
 
 extern void initOptions (void)
 {
+    OptionFiles = stringListNew ();
     verbose ("Setting option defaults\n");
     installHeaderListDefaults ();
-    verbose ("    Installing default language mappings:\n");
+    verbose ("  Installing default language mappings:\n");
     installLanguageMapDefaults ();
+
+    /* always excluded by default */
+    verbose ("  Installing default exclude patterns:\n");
+    processExcludeOption (NULL, "EIFGEN");
+    processExcludeOption (NULL, "SCCS");
+    processExcludeOption (NULL, "RCS");
+    processExcludeOption (NULL, "CVS");
+
 }
 
 extern void freeOptionResources (void)
@@ -1589,6 +1640,7 @@ extern void freeOptionResources (void)
     freeList (&Option.ignore);
     freeList (&Option.headerExt);
     freeList (&Option.etagsInclude);
+    freeList (&OptionFiles);
 }
 
 /* vi:set tabstop=8 shiftwidth=4: */
