@@ -1,7 +1,7 @@
 /*
-*   $Id: options.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: options.h,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -47,35 +47,41 @@ typedef struct sCookedArgs {
     char* item;
 } cookedArgs;
 
+typedef enum eLocate {
+    EX_MIX,		/* line numbers for defines, patterns otherwise */
+    EX_LINENUM,		/* -n  only line numbers in tag file */
+    EX_PATTERN		/* -N  only patterns in tag file */
+} exCmd;
+
+struct sInclude {
+    boolean fileNames;		/* include tags for source file names */
+    boolean qualifiedTags;	/* include tags for qualified class members */
+    boolean fileScope;		/* include tags of file scope only */
+};
+
+struct sExtFields {		/* extension field content control */
+    boolean access;
+    boolean fileScope;
+    boolean implementation;
+    boolean inheritance;
+    boolean kind;
+    boolean kindKey;
+    boolean kindLong;
+    boolean language;
+    boolean lineNumber;
+    boolean scope;
+};
+
 /*  This stores the command line options.
  */
 typedef struct sOptionValues {
-    struct sInclude {
-	boolean fileNames;	/* include tags for source file names */
-	boolean qualifiedTags;	/* include tags for qualified class members */
-	boolean	fileScope;	/* include tags of file scope only */
-    } include;
-    struct sExtFields {		/* extension field content control */
-	boolean access;
-	boolean fileScope;
-	boolean implementation;
-	boolean inheritance;
-	boolean kind;
-	boolean kindKey;
-	boolean kindLong;
-	boolean language;
-	boolean lineNumber;
-	boolean scope;
-    } extensionFields;
+    struct sInclude include;/* --extra  extra tag inclusion */
+    struct sExtFields extensionFields;/* --fields  extension field control */
     stringList* ignore;	    /* -I  name of file containing tokens to ignore */
     boolean append;	    /* -a  append to "tags" file */
     boolean backward;	    /* -B  regexp patterns search backwards */
     boolean etags;	    /* -e  output Emacs style tags file */
-    enum eLocate {
-	EX_MIX,		    /* line numbers for defines, patterns otherwise */
-	EX_LINENUM,	    /* -n  only line numbers in tag file */
-	EX_PATTERN	    /* -N  only patterns in tag file */
-    } locate;		    /* --excmd  EX command used to locate tag */
+    exCmd locate;	    /* --excmd  EX command used to locate tag */
     boolean recurse;	    /* -R  recurse into directories */
     boolean sorted;	    /* -u,--sort  sort tags */
     boolean verbose;	    /* -V  verbose */
@@ -91,7 +97,6 @@ typedef struct sOptionValues {
     boolean followLinks;    /* --link  follow symbolic links? */
     boolean filter;	    /* --filter  behave as filter: files in, tags out */
     char* filterTerminator; /* --filter-terminator  string to output */
-    boolean qualifiedTags;  /* --qualified-tags include class-qualified tag */
     boolean tagRelative;    /* --tag-relative file paths relative to tag file */
     boolean printTotals;    /* --totals  print cumulative statistics */
     boolean lineDirectives; /* --linedirectives  process #line directives */
