@@ -1,7 +1,7 @@
 /*
-*   $Id: parse.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: parse.c,v 1.5 2002/02/19 05:18:46 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -27,6 +27,7 @@
 #include "options.h"
 #include "parsers.h" 
 #include "read.h"
+#include "routines.h"
 #include "vstring.h"
 
 /*
@@ -68,8 +69,15 @@ extern parserDefinition* parserNew (const char* name)
 
 extern const char *getLanguageName (const langType language)
 {
-    Assert (0 <= language  &&  language < (int) LanguageCount);
-    return LanguageTable [language]->name;
+    const char* result;
+    if (language == LANG_IGNORE)
+	result = "unknown";
+    else
+    {
+	Assert (0 <= language  &&  language < (int) LanguageCount);
+	result = LanguageTable [language]->name;
+    }
+    return result;
 }
 
 extern langType getNamedLanguage (const char *const name)
@@ -80,7 +88,7 @@ extern langType getNamedLanguage (const char *const name)
     for (i = 0  ;  i < LanguageCount  &&  result == LANG_IGNORE  ;  ++i)
     {
 	if (LanguageTable [i]->name != NULL)
-	    if (stricmp (name, LanguageTable [i]->name) == 0)
+	    if (strcasecmp (name, LanguageTable [i]->name) == 0)
 		result = i;
     }
     return result;
@@ -125,9 +133,9 @@ static vString* determineInterpreter (const char* const cmd)
     do
     {
 	vStringClear (interpreter);
-	for ( ;  isspace (*p)  ;  ++p)
+	for ( ;  isspace ((int) *p)  ;  ++p)
 	    ; /* no-op */
-	for ( ;  *p != '\0'  &&  ! isspace (*p)  ;  ++p)
+	for ( ;  *p != '\0'  &&  ! isspace ((int) *p)  ;  ++p)
 	    vStringPut (interpreter, (int) *p);
 	vStringTerminate (interpreter);
     } while (strcmp (vStringValue (interpreter), "env") == 0);
@@ -532,8 +540,9 @@ extern void printKindOptions (void)
  "\n  The following options are used to specify which language-specific tag\n\
   types (or kinds) should be included in the tag file. \"Kinds\" is a group of\n\
   one-letter flags designating kinds of tags to either include or exclude from\n\
-  the output. Each letter or group of letters may be preceded by either '+' to\n\
-  add it to those already included, or '-' to exclude it from the output. In\n\
+  the output. Each letter or group of letters may be preceded by either '+' to\n");
+    printf (
+ "  add it to those already included, or '-' to exclude it from the output. In\n\
   the absence of any preceding '+' or '-' sign, only those kinds listed in\n\
   \"kinds\" will be included in the output. Below each option is a list of the\n\
   flags accepted. All kinds are enabled by default unless otherwise noted.\n\n");
