@@ -1,7 +1,7 @@
 /*
-*   $Id: parse.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: parse.h,v 1.7 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -15,7 +15,6 @@
 *   INCLUDE FILES
 */
 #include "general.h"	/* must always come first */
-
 #include "parsers.h"	/* contains list of parsers */
 #include "strlist.h"
 
@@ -39,9 +38,9 @@ typedef void (*parserInitialize) (langType language);
 
 typedef struct sKindOption {
     boolean enabled;			/* are tags for kind enabled? */
-    const int letter;			/* kind letter */
+    int letter;				/* kind letter */
     const char* name;			/* kind name */
-    const char* const description;	/* displayed in --help output */
+    const char* description;		/* displayed in --help output */
 } kindOption;
 
 typedef struct {
@@ -65,6 +64,13 @@ typedef struct {
 
 typedef parserDefinition* (parserDefinitionFunc) (void);
 
+typedef struct {
+    size_t start;	/* character index in line where match starts */
+    size_t length;	/* length of match */
+} regexMatch;
+
+typedef void (*regexCallback) (const char *line, const regexMatch *matches, unsigned int count);
+
 /*
 *   FUNCTION PROTOTYPES
 */
@@ -107,6 +113,8 @@ extern void matchRegex (const vString* const line, const langType language);
 #endif
 extern boolean processRegexOption (const char *const option, const char *const parameter);
 extern void addLanguageRegex (const langType language, const char* const regex);
+extern void addTagRegex (const langType language, const char* const regex, const char* const name, const char* const kinds, const char* const flags);
+extern void addCallbackRegex (const langType language, const char* const regex, const char* flags, const regexCallback callback);
 extern void disableRegexKinds (const langType __unused__ language);
 extern boolean enableRegexKind (const langType language, const int kind, const boolean mode);
 extern void printRegexKindOptions (const langType language);
