@@ -1,7 +1,7 @@
 /*
-*   $Id: parsers.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: parsers.h,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 2000-2001, Darren Hiebert
+*   Copyright (c) 2000-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -27,6 +27,7 @@
     FortranParser, \
     JavaParser, \
     LispParser, \
+    LuaParser, \
     MakefileParser, \
     PascalParser, \
     PerlParser, \
@@ -38,7 +39,8 @@
     ShParser, \
     SlangParser, \
     TclParser, \
-    VimParser
+    VimParser, \
+    YaccParser
 
 #endif	/* _PARSERS_H */
 
