@@ -1,7 +1,7 @@
 /*
-*   $Id: perl.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: perl.c,v 1.2 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 2000-2001, Darren Hiebert
+*   Copyright (c) 2000-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -24,11 +24,13 @@
 *   DATA DEFINITIONS
 */
 typedef enum {
-    K_SUBROUTINE
+    K_SUBROUTINE,
+    K_PACKAGE
 } perlKind;
 
 static kindOption PerlKinds [] = {
-    { TRUE, 's', "subroutine", "subroutines" }
+    { TRUE, 's', "subroutine", "subroutines" },
+    { TRUE, 'p', "package", "packages" }
 };
 
 /*
@@ -44,6 +46,7 @@ static void findPerlTags (void)
     vString *name = vStringNew ();
     boolean skipPodDoc = FALSE;
     const unsigned char *line;
+    perlKind kind;
 
     while ((line = fileReadLine ()) != NULL)
     {
@@ -61,6 +64,8 @@ static void findPerlTags (void)
 			(const char*) line + 1, "cut", (size_t) 3) != 0);
 	    continue;
 	}
+	else if (strcmp ((const char*) line, "__DATA__") == 0)
+	    break;
 	else if (strcmp ((const char*) line, "__END__") == 0)
 	    break;
 	else if (line [0] == '#')
@@ -69,8 +74,20 @@ static void findPerlTags (void)
 	while (isspace (*cp))
 	    cp++;
 
-	if (*cp++ == 's' && *cp++ == 'u' && *cp++ == 'b' && isspace (*cp))
+	if (strncmp((const char*) cp, "sub", (size_t) 3) == 0 ||
+	    strncmp((const char*) cp, "package", (size_t) 7) == 0)
 	{
+	    if (strncmp((const char*) cp, "sub", (size_t) 3) == 0)
+	    {
+	    	cp += 3;
+		kind = K_SUBROUTINE;
+	    } else {
+	    	cp += 7;
+		kind = K_PACKAGE;
+	    }
+	    if (!isspace(*cp))		/* woops, not followed by a space */
+	        continue;
+
 	    while (isspace (*cp))
 		cp++;
 	    while (! isspace ((int) *cp) && *cp != '\0' && *cp != '{' && *cp != '(')
@@ -80,7 +97,7 @@ static void findPerlTags (void)
 	    }
 	    vStringTerminate (name);
 	    if (vStringLength (name) > 0)
-		makeSimpleTag (name, PerlKinds, K_SUBROUTINE);
+		makeSimpleTag (name, PerlKinds, kind);
 	    vStringClear (name);
 	}
     }
