@@ -1,5 +1,5 @@
 /*
- *   $Id: slang.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+ *   $Id: slang.c,v 1.2 2001/12/23 20:31:47 darren Exp $
  *
  *   Copyright (c) 2000-2001, Francesc Rocher
  *
@@ -22,8 +22,12 @@
  */
 static void installSlangRegex (const langType language)
 {
-   addLanguageRegex (language, "/^.*define[ \t]+([A-Z_][A-Z0-9_]*)[^;]*$/\\1/f,function/i");
-   addLanguageRegex (language, "/^[ \t]*implements[ \t]+\\([ \t]*\"([^\"]*)\"[ \t]*\\)[ \t]*;/\\1/n,namespace/");
+   addTagRegex (language,
+	"^.*define[ \t]+([A-Z_][A-Z0-9_]*)[^;]*$",
+	"\\1", "f,function", "i");
+   addTagRegex (language,
+	"^[ \t]*implements[ \t]+\\([ \t]*\"([^\"]*)\"[ \t]*\\)[ \t]*;",
+	"\\1", "n,namespace", NULL);
 }
 
 extern parserDefinition* SlangParser (void)
