@@ -1,7 +1,7 @@
 /*
-*   $Id: sort.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: sort.c,v 1.3 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1996-2001, Darren Hiebert
+*   Copyright (c) 1996-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -22,9 +22,9 @@
 
 #include "debug.h"
 #include "entry.h"
-#include "main.h"
 #include "options.h"
 #include "read.h"
+#include "routines.h"
 #include "sort.h"
 
 #ifdef TRAP_MEMORY_CALLS
