@@ -1,7 +1,7 @@
 /*
-*   $Id: strlist.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: strlist.c,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1999-2001, Darren Hiebert
+*   Copyright (c) 1999-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -20,8 +20,8 @@
 #endif
 
 #include "debug.h"
-#include "main.h"
 #include "read.h"
+#include "routines.h"
 #include "strlist.h"
 
 /*
@@ -84,16 +84,14 @@ extern stringList* stringListNewFromFile (const char* const fileName)
 {
     stringList* result = NULL;
     FILE* const fp = fopen (fileName, "r");
-    if (fp == NULL)
-	error (FATAL | PERROR, "cannot open \"%s\"", fileName);
-    else
+    if (fp != NULL)
     {
 	result = stringListNew ();
 	while (! feof (fp))
 	{
 	    vString* const str = vStringNew ();
 	    readLine (str, fp);
-	    vStringStrip (str);
+	    vStringStripTrailing (str);
 	    if (vStringLength (str) > 0)
 		stringListAdd (result, str);
 	    else
@@ -144,7 +142,8 @@ extern void stringListDelete (stringList *const current)
     }
 }
 
-extern boolean stringListHas (const stringList *const current, const char *const str)
+extern boolean stringListHas (const stringList *const current,
+			      const char *const str)
 {
     boolean result = FALSE;
     unsigned int i;
@@ -161,7 +160,18 @@ extern boolean stringListHasInsensitive (const stringList *const current,
     unsigned int i;
     Assert (current != NULL);
     for (i = 0  ;  ! result  &&  i < current->count  ;  ++i)
-	result = (boolean) (stricmp (str, vStringValue (current->list [i]))==0);
+	result = (boolean) (strcasecmp (str, vStringValue (current->list [i]))==0);
+    return result;
+}
+
+extern boolean stringListHasTest (const stringList *const current,
+				  boolean (*test)(const char *s))
+{
+    boolean result = FALSE;
+    unsigned int i;
+    Assert (current != NULL);
+    for (i = 0  ;  ! result  &&  i < current->count  ;  ++i)
+	result = (*test)(vStringValue (current->list [i]));
     return result;
 }
 
@@ -182,7 +192,7 @@ static boolean fileNameMatched (const vString* const vpattern,
 #if defined (HAVE_FNMATCH)
     return (boolean) (fnmatch (pattern, fileName, 0) == 0);
 #elif defined (CASE_INSENSITIVE_FILENAMES)
-    return (boolean) (stricmp (pattern, fileName) == 0);
+    return (boolean) (strcasecmp (pattern, fileName) == 0);
 #else
     return (boolean) (strcmp (pattern, fileName) == 0);
 #endif
