@@ -1,7 +1,7 @@
 /*
-*   $Id: vstring.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: vstring.h,v 1.2 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -54,7 +54,9 @@ extern void vStringClear (vString *const string);
 extern vString *vStringNew (void);
 extern void vStringDelete (vString *const string);
 extern void vStringPut (vString *const string, const int c);
-extern void vStringStrip (vString *const string);
+extern void vStringStripNewline (vString *const string);
+extern void vStringStripLeading (vString *const string);
+extern void vStringStripTrailing (vString *const string);
 extern void vStringCatS (vString *const string, const char *const s);
 extern void vStringNCatS (vString *const string, const char *const s, const size_t length);
 extern vString *vStringNewCopy (vString *const string);
